;********************************************************************
;--------------------------------------------------------------------
;
;    Titolo        : Ricevitore con 16F5xx e 4 relays.

;    PIC           : 16F505/506/526
;    Supporto      : MPASM
;    Versione      : 1.0
;    Data          : 01-05-2013
;    Ref. hardware :
;    Autore        :  afg
;
;--------------------------------------------------------------------
; Il ricevitore si pone in attesa del prembolo da 10ms.
; Riconosciuto questo, verifica 16 bit seguenti.
; Il contenuto del secondo byte  il complemento del primo.
; Il nibble alto contiene il numero da 0 a F dell'identificativo, che
;  fisso nel firmware.
; Il nibble basso contiene l'indirizzo del realay da azionare.
; Ogni relay pu essere azionato passo-passo o monoistabile 1s in
; dipendenza dello stato di 4 jumper (chiuso = passo passo
; aperto=monostabile).
; Per evitare ripetizioni, la ricezione  sospesa 1s dopo ogni
; acquisizione .
;--------------------------------------------------------------------
;
;  Impiego pin :
;  ----------------
;    16F505/506/526 @ 14 pin
;
;                  |\/|
;             Vdd -|1   14|- Vss
;             RB5 -|2   13|- RB0
;             RB4 -|3   12|- RB1
;        RB3/MCLR -|4   11|- RB2
;             RC5 -|5   10|- RC0
;             RC4 -|6    9|- RC1
;             RC3 -|7    8|- RC2
;                  |______|
;
;    Vdd                     1: +
;    RB5/OSC1/CLKIN          2: In   RxIn
;    RB4/OSC2/CLKOUT         3: In   jumper 4
;    RB3/!MCLR/VPP           4: In   jumper 3
;    RC5/T0CKI               5: Out  nn
;    RC4/[C2OUT]             6: Out  nn
;    RC3                     7: Out  ry 4
;    RC2/[CVref]             8: Out  ry 3
;    RC1/[C2IN-]             9: Out  ry 2
;    RC0/[C2IN+]            10: Out  ry 1
;    RB2/[C1OUT/AN2]        11: Out  nn
;    RB1/[C1IN-/AN1/]ICSPC  12: In   jumper 2
;    RB0/[C1IN+/AN0/]ICSPD  13: In   jumper 1
;    Vss                    14: -
;
;  [] solo 506/526
;********************************************************************
;                      DEFINIZIONE DEL PROCESSORE
 #ifdef __16F505
        LIST p=16F505   r=DEC
        #include <p16F505.inc>
 #endif
 #ifdef __16F506
        LIST p=16F506   r=DEC
        #include <p16F506.inc>
#define  procanalog
 #endif
 #ifdef __16F526
        LIST p=16F526   r=DEC
        #include <p16F526.inc>
#define  procanalog
 #endif

;********************************************************************
;====================================================================
;            DEFINIZIONE DI IMPIEGO DEI PORT
;====================================================================
;
; PORTB map
;|  5  |  4  |  3  |  2  |  1  |  0  |
;|-----|-----|-----|-----|-----|-----|
;|  nn | j4  | j3  | nn  |  j2 | j1  |

#define    j1    PORTB,RB0    ; modo rel 1
#define    j2    PORTB,RB1    ; modo rel 2
;#define       PORTB,RB2      ;
#define    j3    PORTB,RB3    ; modo rel 3
#define    j4    PORTB,RB4    ; modo rel 4
#define    RxIn  PORTB,RB5    ; ingresso rx

; PORTC map
;|  5  |  4  |  3  |  2  |  1  |  0  |
;|-----|-----|-----|-----|-----|-----|
;|  nn | nn  | ry4 | ry3 | ry2 | ry1 |
;
#define    ry1    PORTC,RC0   ; relay 1
#define    ry2    PORTC,RC1   ;       2
#define    ry3    PORTC,RC2   ;       3
#define    ry4    PORTC,RC3   ;       4
;#define       PORTB,RB4      ;
;#define       PORTB,RB5      ;
;
;********************************************************************
;####################################################################
;
;    Note :
;
;********************************************************************

;####################################################################
;====================================================================
;                          CONFIGURAZIONE                           =
;====================================================================
;
; Oscillatore interno, no WDT, no CP, pin4=GP3;
 #ifdef __16F505
 __config  _IntRC_OSC_CLKOUTEN & _WDT_OFF & _CP_OFF & _MCLRE_OFF
 #endif
 #ifdef __16F506
 __config  _IntRC_OSC_CLKOUTEN & _IOSCFS_OFF & _WDT_OFF & _CP_OFF & _MCLRE_OFF
 #endif
 #ifdef __16F526
 __config  _IntRC_OSC_CLKOUT & _IOSCFS_4MHz & _WDTE_OFF & _CP_OFF & _MCLRE_OFF
 #endif

;####################################################################
;====================================================================
;=                        MEMORIA RAM                               =
;====================================================================
; general purpose RAM
        CBLOCK 0x10             ; area RAM comune
 rx_byte1                       ; primo byte ricevuto
 rx_byte2                       ; secondo
 rx_buf                         ; buffer
 bit_cntr                       ; contatore bit ricevuti
 l_status                       ; local status
 d_0                            ; delays
 d_1                            ; delays
 d_2                            ; delays
 d_3                            ; delays
        ENDC

#define  repeat_flg1  l_status,0 ; flag anti repeat_flg 1s
#define  repeat_flg2  l_status,1 ; flag anti repeat_flg 2s

_Identy   EQU    0x50           ; identificativo fisso

;********************************************************************

;********************************************************************
        ORG     0x000

; Reset Vector

; calibrate internal oscillator
         movwf   OSCCAL
         goto    Init

;********************************************************************
;                          SUBROUTINES
;====================================================================

;====================================================================
; Riceve 1 byte
;====================================================================
;
Rx_1byte
        movlw   .8              ; 8 bit
        movwf   bit_cntr
        clrf    rx_buf

rx_1blp
        rlf     rx_buf,f        ; per prossimo bit

        call    Delay_600us     ; avanti di 1/2 bit

        btfss   RxIn            ; in ricevuto = 1 ?
         goto   rx_1bll         ; n - bit = 0

; bit = 1
        bsf     rx_buf,0        ; bit a 1

rx_1blh
        btfsc   RxIn            ; attesa fine bit HIGH
         goto   rx_1blh         ; n - attesa
        goto    rx_1bl0         ; s - avanti

; bit = 0
rx_1bll
        bcf     rx_buf,0

rx_1bl0                         ; attesa fine bit LOW
        btfss   RxIn            ; bit=1 ?
         goto   rx_1bl0         ; n - attesa

; fine bit low - ultimo bit ?
        decfsz  bit_cntr,f      ; fine bit?
         goto   rx_1blp         ; n - ricicla

        retlw   0

;====================================================================
; DELAYS
;====================================================================
; core ((n*3)-1)
Delay_200us             ; 200 cicli
        movlw   .64     ;+1 ((64*3)-1)=191+1+2+1+5
        goto    dlyuc   ;+2
Delay_300us             ; 300 cicli
        movlw   .97     ;+1 ((97*3)-1)=290+1+3+1+5
        nop             ;+3
        goto    dlyuc
Delay_400us             ; 400 cicli
        movlw   .130    ;+1  ((130*3)-1)=389+1+4+1+5
        goto    $+1     ;+4
        goto    dlyuc   ;
Delay_600us             ; 600 cicli
        movlw   .197    ;+1  ((197*3)-1)=590+1+3+1+5
        nop             ;+3
        goto    $+1
dlyuc   movwf   d_1     ;+1
dly_us                  ; |
        decfsz  d_1, f  ; |((n*3)-1)
         goto   dly_us  ; |
        nop             ;5 cicli
        retlw   0       ;

Delay_200ms             ;199993 cycles
        movlw   0x3E
        movwf   d_1
        movlw   0x9D
        goto    dlymc
Delay_10ms              ;9993 cycles
        movlw   0xCE
        movwf   d_1
        movlw   0x08
dlymc   movwf   d_2
dly_ms
        decfsz  d_1, f
         goto   $+2
        decfsz  d_2, f
         goto   dly_ms
        goto    $+1     ;3 cycles
        nop
        retlw   0       ;4 cycles

Delay_1s
        movlw   .5              ; delay 1s
        movwf   d_3
dly_s   call    Delay_200ms
        decfsz  d_3,f
         goto   dly_s
        retlw   0

;====================================================================

;####################################################################
;                        MAIN PROGRAM
Init:
; inizializzazioni al reset
        clrf    PORTB           ; preset PORTB latch a 0
        clrf    PORTC           ; preset PORTC latch a 0

 #ifdef procanalog              ; solo per 16F506/526
; disabilita gli ingressi analogici
        clrf    ADCON0
; disabilita comparatori per liberare la funzione digitale
        movlw   0x77
        movwf   CM1CON0
        movwf   CM2CON0
 #endif

; TRISPORTB       --111111      RB in
        movlw   b'11111111'
        tris    PORTB

; TRISPORTC       --111111      RC5:0 Out
        movlw   b'11000000'
        tris    PORTC

; no T0CKI, Timer0 con prescaler 1:256
      ; default b'11111111'
      ;           1-------     GPWU disabilitato
      ;           -0------     GPPU abilitato (RB4,3,1,0)
      ;           --0-----     clock interno
      ;           ---1----     falling
      ;           ----0---     prescaler al Timer0
      ;           -----101     1:64
        movlw   b'10010101'
        OPTION

        clrf    l_status        ; clear all local flags

        call    Delay_200ms     ; stabilizzazione


;**********************************************************************
main

;---------------------------------------
rxdata
        clrf    rx_byte1
        clrf    rx_byte2

        btfss   repeat_flg2    ; 2s antirepeat?
         goto   rxd1           ; n - jump to test next antirepeat
        call    Delay_1s       ; s - add delay 1s
        goto    add1s          ; add another 1s

rxd1    btfss   repeat_flg1    ; 1s anti repeat?
         goto   intst          ; n - jump to rx
add1s   call    Delay_1s       ; y - add delay 1s

intst   clrf    l_status       ; cancella repeat flags
; attesa in=1
intst1  btfss   RxIn           ; rx =1 ?
         goto   intst1         ; no - attesa preambolo

;---------------------------------------
; verifica se preambolo
; Se rx=1 per 9ms circa passa alla ricezione
        movlw   .116           ; 8,96 ms
        movwf   TMR0
        nop
        nop
phtst
        btfsc   RxIn           ; in=0 ?
         goto   thtst          ; n - timeout ?
        goto    rxdata         ; s - in=0 - abort

thtst   movf    TMR0,w         ; timeout?
        bz      thend          ; s - verifica in
         goto   phtst          ; n - stai nel ciclo

; timeout con in=1 - attesa fine livello alto
thend   btfsc   RxIn
         goto   thend

; timeout con in=0 - attesa fine livello basso
tlend   btfss   RxIn
         goto   tlend

;---------------------------------------
; impulso preambolo terminato - ora ingresso  high
; Leggi pacchetto dati 16 bit
r2B
        call    Rx_1byte        ; dato
        movf    rx_buf,w
        movwf   rx_byte1

        call    Rx_1byte        ; dato complementato
        movf    rx_buf,w
        movwf   rx_byte2

; verifica dato
data_ctr
        movf    rx_byte1,w      ; recupera dato
        andlw   0xF0            ; azzera nibble basso
        xorlw   _Identy         ; identificatore uguale ?
        bnz     rxdata          ; no - abort

; dato 1 ok - verifica secondo byte
        comf    rx_byte2,w      ; decomplementa dato
        xorwf   rx_byte1,w      ; compara
        bnz     rxdata          ; diversi - abort

; byte ricevuti uguali - seleziona indirizzo
        movf    rx_byte1,w      ; ricarica dato
        andlw   0x0F            ; solo indirizzo
        xorlw   1
        bz      case1
        xorlw   1^2
        bz      case2
        xorlw   2^3
        bz      case3
        xorlw   3^4
        bz      case4
         goto   rxdata          ; non valido

case1   movlw   .1              ; 00000001 - bit 0
        movwf   rx_buf
        btfss   j1              ; monostabile se jump libero
         goto   toggleb         ; bistabile se jump inserito
        bsf     ry1
        goto    casem_com
case2   movlw   .2              ; 00000010 - bit 1
        movwf   rx_buf
        btfss   j2
         goto   toggleb
        bsf     ry2
         goto   casem_com
case3   movlw   .4              ; 00000100 - bit 2
        movwf   rx_buf
        btfss   j3
         goto   toggleb
        bsf     ry3
         goto   casem_com
case4   movlw   .8              ; 00001000 - bit 3
        movwf   rx_buf
        btfss   j4
         goto   toggleb
        bsf     ry4
casem_com
        call    Delay_1s        ; mantieni acceso per 1s
        bsf     repeat_flg1     ; set anti repeat 1s
        goto    togl1

toggleb bsf     repeat_flg2     ; set anti repeat 2s
togl1   movf    PORTC,w         ; toggle bit
        xorwf   rx_buf,w
        movwf   PORTC

        goto    rxdata

;********************************************************************
        END






































