;********************************************************************
;--------------------------------------------------------------------
;
;    Titolo        : Tx radio 433, 4 pulsanti

;    PIC           : 16F508/509/510/519
;    Supporto      : MPASM
;    Versione      : 1.0
;    Data          : 01-05-2013
;    Ref. hardware :
;    Autore        : afg
;
;--------------------------------------------------------------------
;  Viene trasmesso un pacchetto dati cos costituito:
; - preambolo a livello 1 per 10ms
; - 1 byte composto da un numero fisso (identificativo) nel nibble
;   alto e un numero dipendente dal tasto premuto (indirizzo) nel
;   nibble basso
; - 1 byte complemento del precedente
;  Segue una attesa di 50ms e il ciclo si ripete fino a che il
;  microcontroller  alimentato.
;  Lo stato dei pulsanti  letto solo all'avvio.
;  Il LED  acceso durante la trasmissione.
;--------------------------------------------------------------------

;  Impiego pin :
;  ----------------
;       12F508/509/510/519 @ 8 pin
;
;                  |\/|
;             Vdd -|1    8|- Vss
;             GP5 -|2    7|- GP0
;             GP4 -|3    6|- GP1
;        GP3/MCLR -|4    5|- GP2
;                  |______|
;
;    Vdd                      1: +Vdd
;    GP5/OSC1/CLKIN           2: Out  Tx
;    GP4/OSC2                 3: Out  LED alla Vss
;    GP3/!MCLR/VPP            4: In   pulsante 4
;    GP2/T0CKI/[AN2/COUT]     5: In   pulsante 3
;    GP1/ICSPCLK/[AN1/C1IN-]  6: In   pulsante 2
;    GP0/ICSPDAT/[AN0/C1IN+]  7: In   pulsante 1
;    Vss                      8: -Vss
;
; [] solo 12F510
;********************************************************************
;            DEFINIZIONE DI IMPIEGO DEI PORT
;
;GPIO map
;|  5  |  4  |  3  |  2  |  1  |  0  |
;|-----|-----|-----|-----|-----|-----|
;|TxOut|LED  |Ad_4 |Ad_3 |Ad_2 |Ad_1 |
;
#define     Ad_1  GPIO,GP0    ; pulsante 1
#define     Ad_2  GPIO,GP1    ; pulsante 2
#define     Ad_3  GPIO,GP2    ; pulsante 3
#define     Ad_4  GPIO,GP3    ; pulsante 4
#define     LED   GPIO,GP4    ; LED tra pin e Vss
#define     OutTx GPIO,GP5    ; uscita Tx
;
;********************************************************************
;####################################################################
;
;    Note :
;
;********************************************************************
;####################################################################
; scelta del processore
 #ifdef  __12F509
         LIST      p=12F509  r=DEC
         #include <p12F509.inc>
 #endif
 #ifdef  __12F508
        LIST      p=12F508   r=DEC
        #include <p12F508.inc>
 #endif
 #ifdef  __12F519
         LIST      p=12F519   r=DEC
         #include <p12F519.inc>
 #endif
 #ifdef  __12F510
        LIST      p=12F510    r=DEC
         #include <p12F510.inc>
#define  procanalog                 ; chip con ADC e comparatore
 #endif

;####################################################################
;====================================================================
;                          CONFIGURAZIONE                           =
;====================================================================
; Oscillatore interno, no WDT, no CP, GP3

 #ifdef  procanalog     ; solo per 12F510
 __config  _IntRC_OSC & _IOSCFS_OFF & _CP_OFF & _WDT_OFF & _MCLRE_OFF
 #else
 ; altri
 __config  _IntRC_OSC & _WDT_OFF & _CP_OFF & _MCLRE_OFF
 #endif

;####################################################################
;====================================================================
;=                        MEMORIA RAM                               =
;====================================================================
; general purpose RAM
        CBLOCK 0x10         ; area RAM comune
  bitcntr      ; contatore bit trasmessi
  Tx_Byte1     ; byte dato
  Tx_Byte2     ; byte dato complementato
  Tx_buf       ; buffer di trasmissione
  d_1          ; contatori per delays
  d_2
        ENDC

Identy   EQU  0x50       ; identificativo fisso

;####################################################################


;####################################################################
;====================================================================
;=                         RESET ENTRY                              =
;====================================================================
; Reset Vector
RESET_VECTOR   ORG     0x00

; calibrazione oscillatore interno
         movwf   OSCCAL
         goto    start

;====================================================================
;                          SUBROUTINES
;====================================================================
;--------------------------------------------------------------------
; DELAYS
;--------------------------------------------------------------------
; core ((n*3)-1)
Delay_200us             ; 200 cicli
        movlw   .64     ;+1 ((64*3)-1)=191+1+2+1+5
        goto    dlyuc   ;+2
Delay_300us             ; 300 cicli
        movlw   .97     ;+1 ((97*3)-1)=290+1+3+1+5
        nop             ;+3
        goto    dlyuc
Delay_400us             ; 400 cicli
        movlw   .130    ;+1  ((130*3)-1)=389+1+4+1+5
        goto    $+1     ;+4
        goto    dlyuc   ;
Delay_600us             ; 600 cicli
        movlw   .197    ;+1  ((197*3)-1)=590+1+3+1+5
        nop             ;+3
        goto    $+1
dlyuc   movwf   d_1     ;+1
dly_us                  ; |
        decfsz  d_1, f  ; |((n*3)-1)
         goto   dly_us  ; |
        nop             ;5 cicli
        retlw   0       ;


Delay_200ms             ;199993 cycles
        movlw   0x3E
        movwf   d_1
        movlw   0x9D
        goto    dlymc
Delay_50ms              ;49993 cycles
        movlw   0x0E
        movwf   d_1
        movlw   0x28
        goto    dlymc
Delay_10ms              ;9993 cycles
        movlw   0xCE
        movwf   d_1
        movlw   0x08
dlymc   movwf   d_2
dly_10ms
        decfsz  d_1, f
        goto    $+2
        decfsz  d_2, f
        goto    dly_10ms
        goto    $+1     ;3 cycles
        nop
        retlw   0       ;4 cycles (including call)

;--------------------------------------------------------------------
; Trasmette 1 byte
;--------------------------------------------------------------------
Tx_Byte
        movwf   Tx_buf         ; restore data

        movlw   08h
        movwf   bitcntr

tx_b0
        bsf     OutTx
        call    Delay_400us    ; 400us on

        btfss   Tx_buf,7       ; se bit= 1
         goto   is0
        call    Delay_400us    ; aggiungi altri 400us on

is0     bcf     OutTx
        call    Delay_400us    ; 400us off

        rlf     Tx_buf,f       ; altro bit

        decfsz  bitcntr,f
         goto   tx_b0

        retlw   0

;====================================================================
; MAIN
start
 #ifdef  procanalog      ; solo per 12F510
; disabilita gi ingressi analogici
        clrf    ADCON0
; disabilita comparatori per liberare la funzione digitale
        bcf     CM1CON0, C1ON
 #endif
        clrf    GPIO

; GPPU, clock interno, T0 @ 1:128
        ;         11010111'
        ;         1-------     GPWU disabilitato
        ;         -0------     GPPU abilitati (0,1,3)
        ;         --0-----     clock interno
        ;         ---1----     done
        ;         ----0---     prescaler al timer
        ;         -----110     1:128
        movlw   b'10010110'
        OPTION

        movlw   b'00001111'    ; GP0,1,2,3 in
        tris    GPIO           ; GP4,5 out

; Ritardo per assestamento alimentazione
        call    Delay_200ms

; MAIN
Main
        movlw   Identy         ; indentificativo nel
        movwf   Tx_Byte1       ; nibble alto del dato

; verifica tasto premuto : pu essere premuto pi di un tasto.
; In tal caso l'indirizzo  la somma del valore dei tasti.
; Se nessun tasto risulta premuto (indirizzo 0) il dato non 
; valido.
Addr1
        btfsc   Ad_1
         goto   Addr2
        incf    Tx_Byte1,f     ; Se jumper Ad_1, TxByte1 + 1
Addr2
        btfsc   Ad_2
         goto   Addr3
        movlw   .2             ; Se jumper Ad_1, TxByte1 + 2
        addwf   Tx_Byte1,f
Addr3
        btfsc   Ad_3
         goto   Addr4
        movlw   .3             ; Se jumper Ad_4, TxByte1 + 3
        addwf   Tx_Byte1,f
Addr4
        btfsc   Ad_4
         goto   btnerr         ; verifica validit tasti
        movlw   .4             ; Se jumper Ad_4, TxByte1 + 4
        addwf   Tx_Byte1,f

btnerr  movlw   Identy         ; escludi indirizzo=0
        xorwf   Tx_Byte1,w
        skpnz
         goto   Main           ; nessuno switch valido, abort

dataok  bsf     LED            ; LED on

Main1
        comf    Tx_Byte1,w     ; crea complemento
        movwf   Tx_Byte2

; *******************************************************************
; 8 bit codificati nella forma:
; - bit=0  400ms on/400ms off
; - bit=1  800ms on/400ms off
;
; Preambolo: 10ms on/400us off
;********************************************************************
Tx
; trasmette preambolo 10ms a livello alto e 400us a livello basso
        bsf     OutTx
        call    Delay_10ms    ; 10ms on
        bcf     OutTx
        call    Delay_400us   ; 400us off

; Invia bytes
        movf    Tx_Byte1,w    ; dato
        call    Tx_Byte

        movf    Tx_Byte2,w    ; dato complementato
        call    Tx_Byte

        bcf     LED           ; LED off

        call    Delay_50ms    ; pausa 50ms off

        goto    dataok        ; loop continuo

; *******************************************************************
        END
